package xrand

import (
	randx "crypto/rand"
	"math"
	"math/big"
	"math/rand"
)

var rd *rand.Rand

func init() {
	rd = rand.New(rand.NewSource(randInt64()))
}

// 用系统熵构建更自然的随机数，比"math/rand"构建的线性同余自然随机数要慢
func randInt64() int64 {
	n, _ := randx.Int(randx.Reader, big.NewInt(math.MaxInt64))
	return n.Int64()
}

// 随机值
func Rand[T int | int32 | int64 | uint32 | uint64]() T {
	return T(randInt64())
}

// 从[0,n)范围里随机值
func RandN[T int | int32 | int64 | uint | uint32 | uint64](n T) T {
	if int64(n) < 1 {
		return 0
	}
	return T(randInt64() % int64(n)) // [0,n) 的随机整数
}

// 从[min,max)范围里随机值
func RandRange[T int | int32 | int64 | uint32 | uint64](min, max T) T {
	if int64(max) < 1 || int64(min) < 1 || max < min {
		return 0
	}
	return RandN[T](max-min) + min
}

func Perm(n int) []int {
	return rd.Perm(n)
}

func Read(p []byte) (n int, err error) {
	return rd.Read(p)
}

func Shuffle[T any](slice []T) {
	rand.Shuffle(len(slice), func(i, j int) {
		slice[i], slice[j] = slice[j], slice[i]
	})
}

const letterBytes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"

func RandomString(n int) string {
	b := make([]byte, n)
	for i := range b {
		b[i] = letterBytes[rd.Intn(len(letterBytes))]
	}
	return string(b)
}
